@@ -32,6 +32,7 @@
 
 #include <test/support/tdb_catch.h>
 #include "test/support/src/helpers.h"
+#include "test/support/src/vfs_helpers.h"
 #include "tiledb/sm/c_api/tiledb.h"
 #include "tiledb/sm/c_api/tiledb_serialization.h"
 #include "tiledb/sm/serialization/query.h"
@@ -51,18 +52,16 @@ struct IncompleteFx {
   const char* DENSE_ARRAY_NAME = "test_async_dense";
   const char* SPARSE_ARRAY_NAME = "test_async_sparse";
 
+  // VFS setup
+  VFSTestSetup vfs_test_setup_;
+
   // TileDB context
   tiledb_ctx_t* ctx_;
-
-  // Serialization parameters
-  bool serialize_ = false;
-  bool refactored_query_v2_ = false;
-  // Buffers to allocate on server side for serialized queries
-  ServerQueryBuffers server_buffers_;
+  std::string sparse_array_uri_;
+  std::string dense_array_uri_;
 
   // Constructors/destructors
   IncompleteFx();
-  ~IncompleteFx();
 
   // Functions
   void create_dense_array();
@@ -79,19 +78,13 @@ struct IncompleteFx {
   void check_sparse_until_complete();
   void check_sparse_unsplittable_overflow();
   void check_sparse_unsplittable_complete();
-  void remove_dense_array();
-  void remove_sparse_array();
-  void remove_array(const std::string& array_name);
   bool is_array(const std::string& array_name);
 };
 
-IncompleteFx::IncompleteFx() {
-  ctx_ = nullptr;
-  REQUIRE(tiledb_ctx_alloc(nullptr, &ctx_) == TILEDB_OK);
-}
-
-IncompleteFx::~IncompleteFx() {
-  tiledb_ctx_free(&ctx_);
+IncompleteFx::IncompleteFx()
+    : ctx_(vfs_test_setup_.ctx_c)
+    , sparse_array_uri_(vfs_test_setup_.array_uri(SPARSE_ARRAY_NAME))
+    , dense_array_uri_(vfs_test_setup_.array_uri(DENSE_ARRAY_NAME)) {
 }
 
 void IncompleteFx::create_dense_array() {
@@ -163,7 +156,7 @@ void IncompleteFx::create_dense_array() {
   CHECK(rc == TILEDB_OK);
 
   // Create array
-  rc = tiledb_array_create(ctx_, DENSE_ARRAY_NAME, array_schema);
+  rc = tiledb_array_create(ctx_, dense_array_uri_.c_str(), array_schema);
   CHECK(rc == TILEDB_OK);
 
   // Clean up
@@ -245,7 +238,7 @@ void IncompleteFx::create_sparse_array() {
   CHECK(rc == TILEDB_OK);
 
   // Create array
-  rc = tiledb_array_create(ctx_, SPARSE_ARRAY_NAME, array_schema);
+  rc = tiledb_array_create(ctx_, sparse_array_uri_.c_str(), array_schema);
   CHECK(rc == TILEDB_OK);
 
   // Clean up
@@ -295,7 +288,7 @@ void IncompleteFx::write_dense_full() {
 
   // Open array
   tiledb_array_t* array;
-  int rc = tiledb_array_alloc(ctx_, DENSE_ARRAY_NAME, &array);
+  int rc = tiledb_array_alloc(ctx_, dense_array_uri_.c_str(), &array);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_array_open(ctx_, array, TILEDB_WRITE);
   CHECK(rc == TILEDB_OK);
@@ -319,13 +312,7 @@ void IncompleteFx::write_dense_full() {
       ctx_, query, attributes[2], buffers[3], &buffer_sizes[3]);
   CHECK(rc == TILEDB_OK);
 
-  rc = submit_query_wrapper(
-      ctx_,
-      DENSE_ARRAY_NAME,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_);
+  rc = tiledb_query_submit_and_finalize(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Close array
@@ -378,7 +365,7 @@ void IncompleteFx::write_sparse_full() {
 
   // Open array
   tiledb_array_t* array;
-  int rc = tiledb_array_alloc(ctx_, SPARSE_ARRAY_NAME, &array);
+  int rc = tiledb_array_alloc(ctx_, sparse_array_uri_.c_str(), &array);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_array_open(ctx_, array, TILEDB_WRITE);
   CHECK(rc == TILEDB_OK);
@@ -409,14 +396,8 @@ void IncompleteFx::write_sparse_full() {
       ctx_, query, attributes[4], buffers[5], &buffer_sizes[4]);
   CHECK(rc == TILEDB_OK);
 
-  rc = submit_query_wrapper(
-      ctx_,
-      SPARSE_ARRAY_NAME,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_);
-  CHECK(rc == TILEDB_OK);
+  rc = tiledb_query_submit_and_finalize(ctx_, query);
+  REQUIRE(rc == TILEDB_OK);
 
   // Close array
   rc = tiledb_array_close(ctx_, array);
@@ -427,21 +408,6 @@ void IncompleteFx::write_sparse_full() {
   tiledb_query_free(&query);
 }
 
-void IncompleteFx::remove_array(const std::string& array_name) {
-  if (!is_array(array_name))
-    return;
-
-  CHECK(tiledb_object_remove(ctx_, array_name.c_str()) == TILEDB_OK);
-}
-
-void IncompleteFx::remove_dense_array() {
-  remove_array(DENSE_ARRAY_NAME);
-}
-
-void IncompleteFx::remove_sparse_array() {
-  remove_array(SPARSE_ARRAY_NAME);
-}
-
 bool IncompleteFx::is_array(const std::string& array_name) {
   tiledb_object_t type = TILEDB_INVALID;
   REQUIRE(tiledb_object_type(ctx_, array_name.c_str(), &type) == TILEDB_OK);
@@ -462,7 +428,7 @@ void IncompleteFx::check_dense_incomplete() {
 
   // Open array
   tiledb_array_t* array;
-  int rc = tiledb_array_alloc(ctx_, DENSE_ARRAY_NAME, &array);
+  int rc = tiledb_array_alloc(ctx_, dense_array_uri_.c_str(), &array);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_array_open(ctx_, array, TILEDB_READ);
   CHECK(rc == TILEDB_OK);
@@ -480,14 +446,7 @@ void IncompleteFx::check_dense_incomplete() {
   REQUIRE(rc == TILEDB_OK);
 
   // Submit query
-  rc = submit_query_wrapper(
-      ctx_,
-      DENSE_ARRAY_NAME,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_,
-      false);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Check status
@@ -497,7 +456,7 @@ void IncompleteFx::check_dense_incomplete() {
   CHECK(status == TILEDB_INCOMPLETE);
 
   // Free/finalize query
-  rc = finalize_query_wrapper(ctx_, DENSE_ARRAY_NAME, &query, serialize_);
+  rc = tiledb_query_finalize(ctx_, query);
   REQUIRE(rc == TILEDB_OK);
 
   // Close array
@@ -528,7 +487,7 @@ void IncompleteFx::check_dense_until_complete() {
 
   // Open array
   tiledb_array_t* array;
-  int rc = tiledb_array_alloc(ctx_, DENSE_ARRAY_NAME, &array);
+  int rc = tiledb_array_alloc(ctx_, dense_array_uri_.c_str(), &array);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_array_open(ctx_, array, TILEDB_READ);
   CHECK(rc == TILEDB_OK);
@@ -546,14 +505,7 @@ void IncompleteFx::check_dense_until_complete() {
   REQUIRE(rc == TILEDB_OK);
 
   // Submit query
-  rc = submit_query_wrapper(
-      ctx_,
-      DENSE_ARRAY_NAME,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_,
-      false);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Check buffer
@@ -568,14 +520,7 @@ void IncompleteFx::check_dense_until_complete() {
   CHECK(status == TILEDB_INCOMPLETE);
 
   // Resubmit query
-  rc = submit_query_wrapper(
-      ctx_,
-      DENSE_ARRAY_NAME,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_,
-      false);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Check status
@@ -589,7 +534,7 @@ void IncompleteFx::check_dense_until_complete() {
   CHECK(buffer_sizes[0] == 2 * sizeof(int));
 
   // Free/finalize query
-  rc = finalize_query_wrapper(ctx_, DENSE_ARRAY_NAME, &query, serialize_);
+  rc = tiledb_query_finalize(ctx_, query);
   REQUIRE(rc == TILEDB_OK);
 
   // Close array
@@ -615,7 +560,7 @@ void IncompleteFx::check_dense_shrink_buffer_size() {
 
   // Open array
   tiledb_array_t* array;
-  int rc = tiledb_array_alloc(ctx_, DENSE_ARRAY_NAME, &array);
+  int rc = tiledb_array_alloc(ctx_, dense_array_uri_.c_str(), &array);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_array_open(ctx_, array, TILEDB_READ);
   CHECK(rc == TILEDB_OK);
@@ -633,14 +578,7 @@ void IncompleteFx::check_dense_shrink_buffer_size() {
   REQUIRE(rc == TILEDB_OK);
 
   // Submit query
-  rc = submit_query_wrapper(
-      ctx_,
-      DENSE_ARRAY_NAME,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_,
-      false);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Check buffer
@@ -661,14 +599,7 @@ void IncompleteFx::check_dense_shrink_buffer_size() {
   CHECK(rc == TILEDB_OK);
 
   // Resubmit query
-  rc = submit_query_wrapper(
-      ctx_,
-      DENSE_ARRAY_NAME,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_,
-      false);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Check status
@@ -681,7 +612,7 @@ void IncompleteFx::check_dense_shrink_buffer_size() {
   CHECK(buffer_a1[0] == 2);
 
   // Free/finalize query
-  rc = finalize_query_wrapper(ctx_, DENSE_ARRAY_NAME, &query, serialize_);
+  rc = tiledb_query_finalize(ctx_, query);
   REQUIRE(rc == TILEDB_OK);
 
   // Close array
@@ -708,7 +639,7 @@ void IncompleteFx::check_dense_unsplittable_overflow() {
 
   // Open array
   tiledb_array_t* array;
-  int rc = tiledb_array_alloc(ctx_, DENSE_ARRAY_NAME, &array);
+  int rc = tiledb_array_alloc(ctx_, dense_array_uri_.c_str(), &array);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_array_open(ctx_, array, TILEDB_READ);
   CHECK(rc == TILEDB_OK);
@@ -729,14 +660,7 @@ void IncompleteFx::check_dense_unsplittable_overflow() {
   REQUIRE(rc == TILEDB_OK);
 
   // Submit query
-  rc = submit_query_wrapper(
-      ctx_,
-      DENSE_ARRAY_NAME,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_,
-      false);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Get status
@@ -749,7 +673,7 @@ void IncompleteFx::check_dense_unsplittable_overflow() {
   CHECK(buffer_sizes[1] == 0);
 
   // Finalize query
-  rc = finalize_query_wrapper(ctx_, DENSE_ARRAY_NAME, &query, serialize_);
+  rc = tiledb_query_finalize(ctx_, query);
   REQUIRE(rc == TILEDB_OK);
 
   // Close array
@@ -776,7 +700,7 @@ void IncompleteFx::check_dense_unsplittable_complete() {
 
   // Open array
   tiledb_array_t* array;
-  int rc = tiledb_array_alloc(ctx_, DENSE_ARRAY_NAME, &array);
+  int rc = tiledb_array_alloc(ctx_, dense_array_uri_.c_str(), &array);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_array_open(ctx_, array, TILEDB_READ);
   CHECK(rc == TILEDB_OK);
@@ -797,14 +721,7 @@ void IncompleteFx::check_dense_unsplittable_complete() {
   REQUIRE(rc == TILEDB_OK);
 
   // Submit query
-  rc = submit_query_wrapper(
-      ctx_,
-      DENSE_ARRAY_NAME,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_,
-      false);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Check buffers
@@ -812,7 +729,7 @@ void IncompleteFx::check_dense_unsplittable_complete() {
   CHECK(!memcmp(buffer_a2_var, c_buffer_a2_var, sizeof(c_buffer_a2_var)));
 
   // Finalize query
-  rc = finalize_query_wrapper(ctx_, DENSE_ARRAY_NAME, &query, serialize_);
+  rc = tiledb_query_finalize(ctx_, query);
   REQUIRE(rc == TILEDB_OK);
 
   // Close array
@@ -838,7 +755,7 @@ void IncompleteFx::check_dense_reset_buffers() {
 
   // Open array
   tiledb_array_t* array;
-  int rc = tiledb_array_alloc(ctx_, DENSE_ARRAY_NAME, &array);
+  int rc = tiledb_array_alloc(ctx_, dense_array_uri_.c_str(), &array);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_array_open(ctx_, array, TILEDB_READ);
   CHECK(rc == TILEDB_OK);
@@ -856,14 +773,7 @@ void IncompleteFx::check_dense_reset_buffers() {
   REQUIRE(rc == TILEDB_OK);
 
   // Submit query
-  rc = submit_query_wrapper(
-      ctx_,
-      DENSE_ARRAY_NAME,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_,
-      false);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Check buffer
@@ -878,14 +788,7 @@ void IncompleteFx::check_dense_reset_buffers() {
   CHECK(status == TILEDB_INCOMPLETE);
 
   // Resubmit query
-  rc = submit_query_wrapper(
-      ctx_,
-      DENSE_ARRAY_NAME,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_,
-      false);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Check status
@@ -899,7 +802,7 @@ void IncompleteFx::check_dense_reset_buffers() {
   CHECK(buffer_sizes[0] == 2 * sizeof(int));
 
   // Finalize query
-  rc = finalize_query_wrapper(ctx_, DENSE_ARRAY_NAME, &query, serialize_);
+  rc = tiledb_query_finalize(ctx_, query);
   REQUIRE(rc == TILEDB_OK);
 
   // Close array
@@ -925,7 +828,7 @@ void IncompleteFx::check_sparse_incomplete() {
 
   // Open array
   tiledb_array_t* array;
-  int rc = tiledb_array_alloc(ctx_, SPARSE_ARRAY_NAME, &array);
+  int rc = tiledb_array_alloc(ctx_, sparse_array_uri_.c_str(), &array);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_array_open(ctx_, array, TILEDB_READ);
   CHECK(rc == TILEDB_OK);
@@ -943,14 +846,7 @@ void IncompleteFx::check_sparse_incomplete() {
   REQUIRE(rc == TILEDB_OK);
 
   // Submit query
-  rc = submit_query_wrapper(
-      ctx_,
-      SPARSE_ARRAY_NAME,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_,
-      false);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Check status
@@ -960,7 +856,7 @@ void IncompleteFx::check_sparse_incomplete() {
   CHECK(status == TILEDB_INCOMPLETE);
 
   // Finalize query
-  rc = finalize_query_wrapper(ctx_, SPARSE_ARRAY_NAME, &query, serialize_);
+  rc = tiledb_query_finalize(ctx_, query);
   REQUIRE(rc == TILEDB_OK);
 
   // Close array
@@ -991,7 +887,7 @@ void IncompleteFx::check_sparse_until_complete() {
 
   // Open array
   tiledb_array_t* array;
-  int rc = tiledb_array_alloc(ctx_, SPARSE_ARRAY_NAME, &array);
+  int rc = tiledb_array_alloc(ctx_, sparse_array_uri_.c_str(), &array);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_array_open(ctx_, array, TILEDB_READ);
   CHECK(rc == TILEDB_OK);
@@ -1009,14 +905,7 @@ void IncompleteFx::check_sparse_until_complete() {
   REQUIRE(rc == TILEDB_OK);
 
   // Submit query
-  rc = submit_query_wrapper(
-      ctx_,
-      SPARSE_ARRAY_NAME,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_,
-      false);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Check status
@@ -1031,14 +920,7 @@ void IncompleteFx::check_sparse_until_complete() {
   CHECK(buffer_sizes[0] == sizeof(int));
 
   // Resubmit the query
-  rc = submit_query_wrapper(
-      ctx_,
-      SPARSE_ARRAY_NAME,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_,
-      false);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Check status
@@ -1060,14 +942,7 @@ void IncompleteFx::check_sparse_until_complete() {
    */
   if (!use_refactored_sparse_global_order_reader()) {
     // Submit query
-    rc = submit_query_wrapper(
-        ctx_,
-        SPARSE_ARRAY_NAME,
-        &query,
-        server_buffers_,
-        serialize_,
-        refactored_query_v2_,
-        false);
+    rc = tiledb_query_submit(ctx_, query);
     CHECK(rc == TILEDB_OK);
 
     // Check status
@@ -1103,7 +978,7 @@ void IncompleteFx::check_sparse_unsplittable_overflow() {
 
   // Open array
   tiledb_array_t* array;
-  int rc = tiledb_array_alloc(ctx_, SPARSE_ARRAY_NAME, &array);
+  int rc = tiledb_array_alloc(ctx_, sparse_array_uri_.c_str(), &array);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_array_open(ctx_, array, TILEDB_READ);
   CHECK(rc == TILEDB_OK);
@@ -1125,14 +1000,7 @@ void IncompleteFx::check_sparse_unsplittable_overflow() {
   REQUIRE(rc == TILEDB_OK);
 
   // Submit query
-  rc = submit_query_wrapper(
-      ctx_,
-      SPARSE_ARRAY_NAME,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_,
-      false);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
   tiledb_query_status_t status;
   rc = tiledb_query_get_status(ctx_, query, &status);
@@ -1140,19 +1008,13 @@ void IncompleteFx::check_sparse_unsplittable_overflow() {
   CHECK(status == TILEDB_INCOMPLETE);
   CHECK(buffer_sizes[0] == 0);
 
-  // For remote arrays the reason is always TILEDB_REASON_USER_BUFFER_SIZE, but
-  // we can't test it here since we simulate "remote" arrays by using a local
-  // URI so the array->is_remote() check will fail, and we won't get the
-  // correct result.
-  if (!serialize_) {
-    tiledb_query_status_details_t details;
-    rc = tiledb_query_get_status_details(ctx_, query, &details);
-    CHECK(rc == TILEDB_OK);
-    CHECK(details.incomplete_reason == TILEDB_REASON_USER_BUFFER_SIZE);
-  }
+  tiledb_query_status_details_t details;
+  rc = tiledb_query_get_status_details(ctx_, query, &details);
+  CHECK(rc == TILEDB_OK);
+  CHECK(details.incomplete_reason == TILEDB_REASON_USER_BUFFER_SIZE);
 
   // Finalize query
-  rc = finalize_query_wrapper(ctx_, SPARSE_ARRAY_NAME, &query, serialize_);
+  rc = tiledb_query_finalize(ctx_, query);
   REQUIRE(rc == TILEDB_OK);
 
   // Close array
@@ -1179,7 +1041,7 @@ void IncompleteFx::check_sparse_unsplittable_complete() {
 
   // Open array
   tiledb_array_t* array;
-  int rc = tiledb_array_alloc(ctx_, SPARSE_ARRAY_NAME, &array);
+  int rc = tiledb_array_alloc(ctx_, sparse_array_uri_.c_str(), &array);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_array_open(ctx_, array, TILEDB_READ);
   CHECK(rc == TILEDB_OK);
@@ -1200,14 +1062,7 @@ void IncompleteFx::check_sparse_unsplittable_complete() {
   REQUIRE(rc == TILEDB_OK);
 
   // Submit query
-  rc = submit_query_wrapper(
-      ctx_,
-      SPARSE_ARRAY_NAME,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_,
-      false);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Check buffers
@@ -1215,7 +1070,7 @@ void IncompleteFx::check_sparse_unsplittable_complete() {
   CHECK(!memcmp(buffer_a2_var, c_buffer_a2_var, sizeof(c_buffer_a2_var)));
 
   // Finalize query
-  rc = finalize_query_wrapper(ctx_, SPARSE_ARRAY_NAME, &query, serialize_);
+  rc = tiledb_query_finalize(ctx_, query);
   REQUIRE(rc == TILEDB_OK);
 
   // Close array
@@ -1230,18 +1085,7 @@ void IncompleteFx::check_sparse_unsplittable_complete() {
 TEST_CASE_METHOD(
     IncompleteFx,
     "C API: Test incomplete read queries, dense",
-    "[capi][incomplete][dense-incomplete][serialization]") {
-  SECTION("no serialization") {
-    serialize_ = false;
-  }
-#ifdef TILEDB_SERIALIZATION
-  SECTION("serialization enabled global order write") {
-    serialize_ = true;
-    refactored_query_v2_ = GENERATE(true, false);
-  }
-#endif
-
-  remove_dense_array();
+    "[capi][incomplete][dense-incomplete][serialization][rest]") {
   create_dense_array();
   write_dense_full();
   check_dense_incomplete();
@@ -1250,54 +1094,25 @@ TEST_CASE_METHOD(
   check_dense_unsplittable_overflow();
   check_dense_unsplittable_complete();
   check_dense_reset_buffers();
-  remove_dense_array();
 }
 
 TEST_CASE_METHOD(
     IncompleteFx,
     "C API: Test incomplete read queries, sparse",
-    "[capi][incomplete][sparse][serialization]") {
-  SECTION("no serialization") {
-    serialize_ = false;
-  }
-#ifdef TILEDB_SERIALIZATION
-  SECTION("serialization enabled global order write") {
-    serialize_ = true;
-    refactored_query_v2_ = GENERATE(true, false);
-  }
-#endif
-
-  remove_sparse_array();
+    "[capi][incomplete][sparse][serialization][rest]") {
   create_sparse_array();
   write_sparse_full();
   check_sparse_incomplete();
   check_sparse_until_complete();
   check_sparse_unsplittable_overflow();
   check_sparse_unsplittable_complete();
-  remove_sparse_array();
 }
 
-#ifdef TILEDB_SERIALIZATION
-
 TEST_CASE_METHOD(
     IncompleteFx,
     "C API: Test incomplete read queries, dense, serialized",
-    "[capi][incomplete][dense][serialization]") {
-  SECTION("no serialization") {
-    serialize_ = false;
-  }
-#ifdef TILEDB_SERIALIZATION
-  SECTION("serialization enabled global order write") {
-    serialize_ = true;
-    refactored_query_v2_ = GENERATE(true, false);
-  }
-#endif
-
-  remove_dense_array();
+    "[capi][incomplete][dense][serialization][rest]") {
   create_dense_array();
   write_dense_full();
   check_dense_incomplete();
-  remove_dense_array();
 }
-
-#endif
