@@ -45,6 +45,7 @@
 #include "tiledb/api/c_api/context/context_api_external.h"
 #include "tiledb/api/c_api/context/context_api_internal.h"
 #include "tiledb/common/logger.h"
+#include "tiledb/common/memory_tracker.h"
 #include "tiledb/common/stdx_string.h"
 #include "tiledb/sm/c_api/tiledb_struct_def.h"
 #include "tiledb/sm/cpp_api/tiledb"
@@ -56,6 +57,14 @@
 #include "tiledb/sm/serialization/array.h"
 #include "tiledb/sm/serialization/query.h"
 
+int setenv_local(const char* __name, const char* __value) {
+#ifdef _WIN32
+  return _putenv_s(__name, __value);
+#else
+  return ::setenv(__name, __value, 1);
+#endif
+}
+
 std::mutex catch2_macro_mutex;
 
 namespace tiledb::test {
@@ -299,10 +308,10 @@ void check_subarray(
   uint64_t dim_range_num = 0;
   const type::Range* range;
   for (unsigned i = 0; i < dim_num; ++i) {
-    CHECK(subarray.get_range_num(i, &dim_range_num).ok());
+    CHECK_NOTHROW(subarray.get_range_num(i, &dim_range_num));
     CHECK(dim_range_num == ranges[i].size() / 2);
     for (uint64_t j = 0; j < dim_range_num; ++j) {
-      CHECK(subarray.get_range(i, j, &range).ok());
+      CHECK_NOTHROW(subarray.get_range(i, j, &range));
       auto r = (const T*)range->data();
 
       CHECK(r[0] == ranges[i][2 * j]);
@@ -350,71 +359,32 @@ void check_subarray(
   }
 }
 
-template <class T>
-void check_subarray_equiv(
-    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2) {
-  CHECK(subarray1.range_num() == subarray2.range_num());
-  // Check dim num
-  auto dim_num1 = subarray1.dim_num();
-  auto dim_num2 = subarray2.dim_num();
-  CHECK(dim_num1 == dim_num2);
-
-  tiledb::sm::ByteVec sa1bytes, sa2bytes;
-  //.to_byte_vect() only valid when range_num() == 1, but should be same for
-  // both and resulting bytes, empty or otherwise, should be the same as well.
-  CHECK(
-      subarray1.to_byte_vec(&sa1bytes).ok() ==
-      subarray2.to_byte_vec(&sa2bytes).ok());
-  CHECK(sa1bytes == sa2bytes);
-
-  const std::vector<std::vector<uint8_t>> sa1tilecoords =
-      subarray1.tile_coords();
-  const std::vector<std::vector<uint8_t>> sa2tilecoords =
-      subarray2.tile_coords();
-  CHECK(sa1tilecoords == sa2tilecoords);
-
-  // Check ranges
-  uint64_t dim_range_num1 = 0;
-  const type::Range* range1;
-  uint64_t dim_range_num2 = 0;
-  const type::Range* range2;
-  if (dim_num1 == dim_num2) {
-    for (unsigned i = 0; i < dim_num1; ++i) {
-      CHECK(subarray1.get_range_num(i, &dim_range_num1).ok());
-      CHECK(subarray2.get_range_num(i, &dim_range_num2).ok());
-      CHECK(dim_range_num1 == dim_range_num2);
-      if (dim_range_num1 == dim_range_num2) {
-        for (uint64_t j = 0; j < dim_range_num1; ++j) {
-          CHECK(subarray1.get_range(i, j, &range1).ok());
-          CHECK(subarray2.get_range(i, j, &range2).ok());
-          auto r1 = (const T*)range1->data();
-          auto r2 = (const T*)range2->data();
-          CHECK(r1[0] == r2[0]);
-          CHECK(r1[1] == r2[1]);
-        }
-      }
-    }
-  }
-}
-
 template <class T>
 bool subarray_equiv(
     tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2) {
   bool equiv_state = 1;  // assume true
-
-  equiv_state &= (subarray1.range_num() == subarray2.range_num());
+  auto rn1 = subarray1.range_num();
+  auto rn2 = subarray2.range_num();
+  if (rn1 != rn2) {
+    return false;
+  }
   // Check dim num
   auto dim_num1 = subarray1.dim_num();
   auto dim_num2 = subarray2.dim_num();
-  equiv_state &= (dim_num1 == dim_num2);
+  if (dim_num1 != dim_num2) {
+    return false;
+  }
 
+  /*
+   * `to_byte_vect()` is only valid when `range_num() == 1`, so only
+   * convert in that case, since `to_byte_vect()` will throw otherwise
+   */
   tiledb::sm::ByteVec sa1bytes, sa2bytes;
-  //.to_byte_vect() only valid when range_num() == 1, but should be same for
-  // both and resulting bytes, empty or otherwise, should be the same as well.
-  equiv_state &=
-      (subarray1.to_byte_vec(&sa1bytes).ok() ==
-       subarray2.to_byte_vec(&sa2bytes).ok());
-  equiv_state &= (sa1bytes == sa2bytes);
+  if (rn1 == 1) {
+    subarray1.to_byte_vec(&sa1bytes);
+    subarray2.to_byte_vec(&sa2bytes);
+    equiv_state &= (sa1bytes == sa2bytes);
+  }
 
   const std::vector<std::vector<uint8_t>> sa1tilecoords =
       subarray1.tile_coords();
@@ -429,13 +399,17 @@ bool subarray_equiv(
   const type::Range* range2;
   if (dim_num1 == dim_num2) {
     for (unsigned i = 0; i < dim_num1; ++i) {
-      equiv_state &= (subarray1.get_range_num(i, &dim_range_num1).ok());
-      equiv_state &= (subarray2.get_range_num(i, &dim_range_num2).ok());
+      try {
+        subarray1.get_range_num(i, &dim_range_num1);
+        subarray2.get_range_num(i, &dim_range_num2);
+      } catch (...) {
+        return false;
+      }
       equiv_state &= (dim_range_num1 == dim_range_num2);
       if (dim_range_num1 == dim_range_num2) {
         for (uint64_t j = 0; j < dim_range_num1; ++j) {
-          CHECK(subarray1.get_range(i, j, &range1).ok());
-          CHECK(subarray2.get_range(i, j, &range2).ok());
+          CHECK_NOTHROW(subarray1.get_range(i, j, &range1));
+          CHECK_NOTHROW(subarray2.get_range(i, j, &range2));
           auto r1 = (const T*)range1->data();
           auto r2 = (const T*)range2->data();
           equiv_state &= (r1[0] == r2[0]);
@@ -787,54 +761,27 @@ void get_supported_fs(
     bool* s3_supported,
     bool* hdfs_supported,
     bool* azure_supported,
-    bool* gcs_supported) {
+    bool* gcs_supported,
+    bool* rest_s3_supported) {
   // Override VFS support if the user used the '--vfs' command line argument.
   if (g_vfs.empty()) {
     *s3_supported = tiledb::sm::filesystem::s3_enabled;
     *hdfs_supported = tiledb::sm::filesystem::hdfs_enabled;
     *azure_supported = tiledb::sm::filesystem::azure_enabled;
     *gcs_supported = tiledb::sm::filesystem::gcs_enabled;
+    *rest_s3_supported = false;
   } else {
     if (!(g_vfs == "native" || g_vfs == "s3" || g_vfs == "hdfs" ||
-          g_vfs == "azure" || g_vfs == "gcs")) {
+          g_vfs == "azure" || g_vfs == "gcs" || g_vfs == "rest-s3")) {
       throw std::runtime_error(
           "Failed to get supported fs. Invalid --vfs command line argument.");
     }
 
-    if (g_vfs == "native") {
-      *s3_supported = false;
-      *hdfs_supported = false;
-      *azure_supported = false;
-      *gcs_supported = false;
-    }
-
-    if (g_vfs == "s3") {
-      *s3_supported = true;
-      *hdfs_supported = false;
-      *azure_supported = false;
-      *gcs_supported = false;
-    }
-
-    if (g_vfs == "hdfs") {
-      *s3_supported = false;
-      *hdfs_supported = true;
-      *azure_supported = false;
-      *gcs_supported = false;
-    }
-
-    if (g_vfs == "azure") {
-      *s3_supported = false;
-      *hdfs_supported = false;
-      *azure_supported = true;
-      *gcs_supported = false;
-    }
-
-    if (g_vfs == "gcs") {
-      *s3_supported = false;
-      *hdfs_supported = false;
-      *azure_supported = false;
-      *gcs_supported = true;
-    }
+    *s3_supported = g_vfs == "s3";
+    *hdfs_supported = g_vfs == "hdfs";
+    *azure_supported = g_vfs == "azure";
+    *gcs_supported = g_vfs == "gcs";
+    *rest_s3_supported = g_vfs == "rest-s3";
   }
 }
 
@@ -858,7 +805,7 @@ void create_subarray(
     auto dim_range_num = ranges[d].size() / 2;
     for (size_t j = 0; j < dim_range_num; ++j) {
       type::Range range(&ranges[d][2 * j], 2 * sizeof(T));
-      CHECK(ret.add_range(d, std::move(range), true).ok());
+      CHECK_NOTHROW(ret.add_range(d, std::move(range), true));
     }
   }
 
@@ -1285,8 +1232,7 @@ void read_array(
   tiledb_query_free(&query);
 }
 
-int32_t num_commits(const std::string& array_name) {
-  Context ctx;
+int32_t num_commits(Context ctx, const std::string& array_name) {
   VFS vfs(ctx);
 
   // Get the number of write files in the commit directory
@@ -1294,8 +1240,13 @@ int32_t num_commits(const std::string& array_name) {
   return commits_dir.file_count(sm::constants::write_file_suffix);
 }
 
-int32_t num_fragments(const std::string& array_name) {
+int32_t num_commits(const std::string& array_name) {
   Context ctx;
+
+  return num_commits(ctx, array_name);
+}
+
+int32_t num_fragments(Context ctx, const std::string& array_name) {
   VFS vfs(ctx);
 
   // Get all URIs in the array directory
@@ -1304,6 +1255,12 @@ int32_t num_fragments(const std::string& array_name) {
   return static_cast<uint32_t>(uris.size());
 }
 
+int32_t num_fragments(const std::string& array_name) {
+  Context ctx;
+
+  return num_fragments(ctx, array_name);
+}
+
 std::string random_string(const uint64_t l) {
   static const char char_set[] =
       "0123456789"
@@ -1490,450 +1447,6 @@ int deserialize_array_and_query(
       ctx, c_buff, TILEDB_CAPNP, clientside ? 1 : 0, array_uri, query, &array);
 }
 
-int array_open_wrapper(
-    tiledb_ctx_t* client_ctx,
-    tiledb_query_type_t query_type,
-    bool serialize,
-    tiledb_array_t** open_array) {
-  // Open array
-#ifndef TILEDB_SERIALIZATION
-  return tiledb_array_open(client_ctx, *open_array, query_type);
-#endif
-
-  if (!serialize) {
-    return tiledb_array_open(client_ctx, *open_array, query_type);
-  }
-
-  // Get the config to use it for creating fresh contexts
-  tiledb_config_t* config;
-  int rc = tiledb_ctx_get_config(client_ctx, &config);
-  REQUIRE(rc == TILEDB_OK);
-
-  // this helper only applies to refactored array open, so set it in the config
-  // Note: we actually set use_refactored_array_open_and_query_submit instead of
-  // simple use_refactored_array_open here, because we want array_open_wrapper
-  // to be usable in query_v3 tests that require that flag to be set right
-  // from the beginning for full Array objects to be retrieved on array open.
-  // There are dedicated tests in unit-capi-array.cc that are testing array v2
-  // feature with just setting use_refactored_array_open config variable.
-  tiledb_error_t* error = nullptr;
-  rc = tiledb_config_set(
-      config,
-      "rest.use_refactored_array_open_and_query_submit",
-      "true",
-      &error);
-  REQUIRE(rc == TILEDB_OK);
-  REQUIRE(error == nullptr);
-  REQUIRE(
-      tiledb_array_set_config(client_ctx, *open_array, config) == TILEDB_OK);
-
-  auto serialization_type =
-      (tiledb_serialization_type_t)tiledb::sm::SerializationType::CAPNP;
-
-  // 1. Client: Serialize array_open request to Server. First set the query_type
-  // that will be serialized
-  auto type = static_cast<sm::QueryType>(query_type);
-  (*open_array)->array_->set_query_type(type);
-
-  tiledb_buffer_t* buffer;
-  rc = tiledb_serialize_array_open(
-      client_ctx, *open_array, serialization_type, 1, &buffer);
-  REQUIRE(rc == TILEDB_OK);
-
-  // 2. Server : Receive and deserialize array_open_request
-  // Allocate new context for server
-  tiledb_ctx_t* server_ctx;
-  tiledb_ctx_alloc(config, &server_ctx);
-  tiledb_array_t* deserialized_array_server = nullptr;
-  rc = tiledb_deserialize_array_open(
-      server_ctx, buffer, serialization_type, 0, &deserialized_array_server);
-  REQUIRE(rc == TILEDB_OK);
-  tiledb_buffer_free(&buffer);
-
-  // Check that the original and de-serialized array have the same query type
-  REQUIRE(deserialized_array_server->array_->get_query_type() == type);
-
-  // 3. Server: Open the array the request was received for in the requested
-  // mode
-  // This is needed in test, as the deserialized array has a dummy array_uri of
-  // "deserialized_array" set instead of the original one. The Cloud side
-  // already knows the array URI so it's not a problem in the real life scenario
-  deserialized_array_server->array_->set_array_uri(
-      (*open_array)->array_->array_uri());
-
-  // if we need to add start and end timestamps uncomment this code
-  // rc = tiledb_array_set_open_timestamp_start(server_ctx,
-  // deserialized_array_server, array->array_->timestamp_start());
-  // REQUIRE_SAFE(rc == TILEDB_OK);
-  // rc = tiledb_array_set_open_timestamp_end(server_ctx,
-  // deserialized_array_server, array->array_->timestamp_end()); REQUIRE_SAFE(rc
-  // == TILEDB_OK);
-  rc = tiledb_array_open(server_ctx, deserialized_array_server, query_type);
-  REQUIRE(rc == TILEDB_OK);
-
-  // 4. Server -> Client: Send opened Array (serialize)
-  tiledb_buffer_t* buff;
-  rc = tiledb_serialize_array(
-      server_ctx,
-      deserialized_array_server,
-      (tiledb_serialization_type_t)tiledb::sm::SerializationType::CAPNP,
-      1,
-      &buff);
-  REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_array_close(server_ctx, deserialized_array_server);
-  CHECK(rc == TILEDB_OK);
-  tiledb_array_free(&deserialized_array_server);
-  tiledb_ctx_free(&server_ctx);
-  CHECK(server_ctx == nullptr);
-
-  // 5. Client: Receive and deserialize Array (into deserialized_array_client),
-  // in the same way that rest_client does.
-  auto st = tiledb::sm::serialization::array_deserialize(
-      (*open_array)->array_.get(),
-      tiledb::sm::SerializationType::CAPNP,
-      buff->buffer(),
-      client_ctx->storage_manager());
-  REQUIRE(st.ok());
-
-  // 6. Server: Close array and clean up
-  tiledb_buffer_free(&buff);
-
-  return rc;
-}
-
-int submit_query_wrapper(
-    const Context& client_ctx,
-    const std::string& array_uri,
-    Query* query,
-    ServerQueryBuffers& buffers,
-    bool serialize_query,
-    bool refactored_query_v2,
-    bool finalize) {
-  auto query_c = query->ptr().get();
-  return submit_query_wrapper(
-      client_ctx.ptr().get(),
-      array_uri,
-      &query_c,
-      buffers,
-      serialize_query,
-      refactored_query_v2,
-      finalize);
-}
-
-int submit_query_wrapper(
-    tiledb_ctx_t* client_ctx,
-    const std::string& array_uri,
-    tiledb_query_t** query,
-    ServerQueryBuffers& buffers,
-    bool serialize,
-    bool refactored_query_v2,
-    bool finalize) {
-  int rc = 0;
-#ifndef TILEDB_SERIALIZATION
-  rc = tiledb_query_submit(client_ctx, *query);
-  if (rc != TILEDB_OK) {
-    return rc;
-  }
-
-  if (finalize) {
-    // Finalize query
-    rc = tiledb_query_finalize(client_ctx, *query);
-  }
-
-  return rc;
-#endif
-
-  if (!serialize) {
-    rc = tiledb_query_submit(client_ctx, *query);
-    if (rc != TILEDB_OK) {
-      return rc;
-    }
-
-    if (finalize) {
-      // Finalize query
-      rc = tiledb_query_finalize(client_ctx, *query);
-    }
-
-    return rc;
-  }
-
-  // Get the config to use it for creating fresh contexts
-  tiledb_config_t* config;
-  rc = tiledb_query_get_config(client_ctx, *query, &config);
-  CHECK(rc == TILEDB_OK);
-
-  // Get the array of that query
-  tiledb_array_t* array;
-  REQUIRE(tiledb_query_get_array(client_ctx, *query, &array) == TILEDB_OK);
-
-  // if refactored query v2 set it in the config
-  if (refactored_query_v2) {
-    tiledb_error_t* error = nullptr;
-    rc = tiledb_config_set(
-        config,
-        "rest.use_refactored_array_open_and_query_submit",
-        "true",
-        &error);
-    REQUIRE(rc == TILEDB_OK);
-    REQUIRE(error == nullptr);
-
-    REQUIRE(tiledb_array_close(client_ctx, array) == TILEDB_OK);
-    REQUIRE(tiledb_array_set_config(client_ctx, array, config) == TILEDB_OK);
-    tiledb_query_type_t query_type;
-    REQUIRE_SAFE(
-        tiledb_query_get_type(client_ctx, *query, &query_type) == TILEDB_OK);
-    REQUIRE(tiledb_array_open(client_ctx, array, query_type) == TILEDB_OK);
-  }
-
-  // Get the query type
-  tiledb_query_type_t query_type;
-  REQUIRE_SAFE(
-      tiledb_query_get_type(client_ctx, *query, &query_type) == TILEDB_OK);
-
-  // 1. Client -> Server : Serialize and send query request
-  tiledb_query_t* server_deser_query;
-  std::vector<uint8_t> serialized;
-
-  rc = serialize_query(client_ctx, *query, &serialized, 1);
-  REQUIRE(rc == TILEDB_OK);
-
-  // 2. Server: Deserialize query request
-  // Allocate new context for server
-  tiledb_ctx_t* server_ctx;
-  tiledb_ctx_alloc(config, &server_ctx);
-  tiledb_array_t* array_server = nullptr;
-  if (refactored_query_v2) {
-    rc = deserialize_array_and_query(
-        server_ctx, serialized, &server_deser_query, array_uri.c_str(), 0);
-    REQUIRE_SAFE(rc == TILEDB_OK);
-  } else {
-    // Open array
-    int rc = tiledb_array_alloc(server_ctx, array_uri.c_str(), &array_server);
-    REQUIRE_SAFE(rc == TILEDB_OK);
-
-    rc = tiledb_array_set_open_timestamp_start(
-        server_ctx, array_server, array->array_->timestamp_start());
-    REQUIRE_SAFE(rc == TILEDB_OK);
-    rc = tiledb_array_set_open_timestamp_end(
-        server_ctx, array_server, array->array_->timestamp_end_opened_at());
-    REQUIRE_SAFE(rc == TILEDB_OK);
-
-    rc = tiledb_array_open(server_ctx, array_server, query_type);
-    REQUIRE_SAFE(rc == TILEDB_OK);
-
-    rc = tiledb_query_alloc(
-        server_ctx, array_server, query_type, &server_deser_query);
-    REQUIRE_SAFE(rc == TILEDB_OK);
-    rc = tiledb::test::deserialize_query(
-        server_ctx, serialized, server_deser_query, 0);
-    REQUIRE_SAFE(rc == TILEDB_OK);
-  }
-
-  // This is a feature of the server, not a bug, quoting from query_from_capnp:
-  // "On reads, just set null pointers with accurate size so that the
-  // server can introspect and allocate properly sized buffers separately."
-  // Empty buffers will naturally break query_submit so to go on in test we
-  // need to allocate here as if we were the server.
-  if (query_type == TILEDB_READ) {
-    allocate_query_buffers_server_side(server_ctx, server_deser_query, buffers);
-  }
-
-  // 3. Server: Submit query WITHOUT re-opening the array, using under the hood
-  // the array found in the deserialized query
-  rc = tiledb_query_submit(server_ctx, server_deser_query);
-  if (rc != TILEDB_OK) {
-    return rc;
-  }
-
-  if (finalize) {
-    tiledb_query_status_t status;
-    rc = tiledb_query_get_status(server_ctx, server_deser_query, &status);
-    CHECK(status == TILEDB_COMPLETED);
-
-    rc = tiledb_query_finalize(server_ctx, server_deser_query);
-    if (rc != TILEDB_OK) {
-      return rc;
-    }
-  }
-
-  // Serialize the new query and "send it over the network" (server-side)
-  // 4. Server -> Client : Send query response
-  std::vector<uint8_t> serialized2;
-  rc = serialize_query(server_ctx, server_deser_query, &serialized2, 0);
-  if (rc != TILEDB_OK) {
-    return rc;
-  }
-
-  if (!refactored_query_v2) {
-    // Close array and clean up
-    rc = tiledb_array_close(server_ctx, array_server);
-    CHECK(rc == TILEDB_OK);
-    tiledb_array_free(&array_server);
-  }
-
-  tiledb_ctx_free(&server_ctx);
-  CHECK(server_ctx == nullptr);
-
-  // 5. Client: Deserialize query
-  rc = deserialize_query(client_ctx, serialized2, *query, 1);
-  REQUIRE(rc == TILEDB_OK);
-
-  // Clean up.
-  tiledb_query_free(&server_deser_query);
-
-  return rc;
-}
-
-int finalize_query_wrapper(
-    const Context& client_ctx,
-    const std::string& array_uri,
-    Query* query,
-    bool serialize_query) {
-  auto query_c = query->ptr().get();
-  return finalize_query_wrapper(
-      client_ctx.ptr().get(), array_uri, &query_c, serialize_query);
-}
-
-int finalize_query_wrapper(
-    tiledb_ctx_t* client_ctx,
-    const std::string& array_uri,
-    tiledb_query_t** query,
-    bool serialize) {
-#ifndef TILEDB_SERIALIZATION
-  return tiledb_query_finalize(client_ctx, *query);
-#endif
-
-  if (!serialize) {
-    return tiledb_query_finalize(client_ctx, *query);
-  }
-
-  // Get the config to use it for creating fresh contexts
-  tiledb_config_t* config;
-  auto rc = tiledb_query_get_config(client_ctx, *query, &config);
-  CHECK(rc == TILEDB_OK);
-
-  // Get the query type
-  tiledb_query_type_t query_type;
-  REQUIRE_SAFE(
-      tiledb_query_get_type(client_ctx, *query, &query_type) == TILEDB_OK);
-
-  // 1. Client -> Server : Send query request
-  std::vector<uint8_t> serialized;
-
-  rc = serialize_query(client_ctx, *query, &serialized, 1);
-  REQUIRE(rc == TILEDB_OK);
-
-  // 2. Server: Deserialize query request
-  // Allocate new context for server
-  tiledb_ctx_t* server_ctx;
-  tiledb_ctx_alloc(config, &server_ctx);
-  tiledb_query_t* server_deser_query;
-  rc = deserialize_array_and_query(
-      server_ctx, serialized, &server_deser_query, array_uri.c_str(), 0);
-
-  // 3. Server: Finalize query
-  rc = tiledb_query_finalize(server_ctx, server_deser_query);
-  if (rc != TILEDB_OK) {
-    return rc;
-  }
-
-  /*
-   * TODO: This is disabled because it fails today as the previous serialization
-   * step before finalize, sets the query buffers to NULL for READ queries, so
-   * the following serialization step will hit an assert because of empty
-   * buffers, as query_finalize is a NOOP for READ queries.
-   *
-   * // Serialize the new query and "send it over the network" (server-side)
-   * 3. Server -> Client : Send query response
-   * std::vector<uint8_t> serialized2;
-   * rc = tiledb_query_v2_serialize(
-   *     ctx, array_uri.c_str(), serialized, false, server_deser_query, query);
-   * REQUIRE_SAFE(rc == TILEDB_OK);
-   */
-
-  // Clean up.
-  tiledb_query_free(&server_deser_query);
-
-  return rc;
-}
-
-void allocate_query_buffers_server_side(
-    tiledb_ctx_t* ctx,
-    tiledb_query_t* query,
-    ServerQueryBuffers& query_buffers) {
-  int rc = 0;
-  auto buffer_names = query->query_->buffer_names();
-  const auto aggregate_names = query->query_->aggregate_buffer_names();
-  buffer_names.insert(
-      buffer_names.end(), aggregate_names.begin(), aggregate_names.end());
-
-  for (uint64_t i = 0; i < buffer_names.size(); i++) {
-    const auto& name = buffer_names[i];
-    const auto& buff = query->query_->buffer(name);
-    const auto& schema = query->query_->array_schema();
-
-    // TODO: This is yet another instance where there needs to be a common
-    // mechanism for reporting the common properties of a field.
-    // Refactor to use query_field_t.
-    bool var_size = false;
-    bool nullable = false;
-    if (query->query_->is_aggregate(name)) {
-      var_size =
-          query->query_->get_aggregate(name).value()->aggregation_var_sized();
-      nullable =
-          query->query_->get_aggregate(name).value()->aggregation_nullable();
-    } else {
-      var_size = schema.var_size(name);
-      nullable = schema.is_nullable(name);
-    }
-
-    if (var_size && buff.buffer_var_ == nullptr) {
-      // Variable-sized buffer
-      query_buffers.attr_or_dim_data.emplace_back(*buff.buffer_var_size_);
-      query_buffers.attr_or_dim_off.emplace_back(
-          (*buff.buffer_size_) / sizeof(sm::constants::cell_var_offset_size));
-      rc = tiledb_query_set_data_buffer(
-          ctx,
-          query,
-          name.c_str(),
-          query_buffers.attr_or_dim_data.back().data(),
-          buff.buffer_var_size_);
-      REQUIRE_SAFE(rc == TILEDB_OK);
-      rc = tiledb_query_set_offsets_buffer(
-          ctx,
-          query,
-          name.c_str(),
-          query_buffers.attr_or_dim_off.back().data(),
-          buff.buffer_size_);
-      REQUIRE_SAFE(rc == TILEDB_OK);
-    }
-
-    if (name == TILEDB_COORDS || (!var_size && buff.buffer_ == nullptr)) {
-      // Fixed-length buffer or Coords
-      query_buffers.attr_or_dim.emplace_back(*buff.buffer_size_);
-      rc = tiledb_query_set_data_buffer(
-          ctx,
-          query,
-          name.c_str(),
-          query_buffers.attr_or_dim.back().data(),
-          buff.buffer_size_);
-      REQUIRE_SAFE(rc == TILEDB_OK);
-    }
-
-    if (nullable) {
-      // nullable
-      query_buffers.attr_or_dim_nullable.emplace_back(
-          *buff.validity_vector_.buffer_size());
-      rc = tiledb_query_set_validity_buffer(
-          ctx,
-          query,
-          name.c_str(),
-          query_buffers.attr_or_dim_nullable.back().data(),
-          buff.validity_vector_.buffer_size());
-    }
-  }
-}
-
 template void check_subarray<int8_t>(
     tiledb::sm::Subarray& subarray, const SubarrayRanges<int8_t>& ranges);
 template void check_subarray<uint8_t>(
@@ -1955,27 +1468,6 @@ template void check_subarray<float>(
 template void check_subarray<double>(
     tiledb::sm::Subarray& subarray, const SubarrayRanges<double>& ranges);
 
-template void check_subarray_equiv<int8_t>(
-    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
-template void check_subarray_equiv<uint8_t>(
-    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
-template void check_subarray_equiv<int16_t>(
-    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
-template void check_subarray_equiv<uint16_t>(
-    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
-template void check_subarray_equiv<int32_t>(
-    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
-template void check_subarray_equiv<uint32_t>(
-    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
-template void check_subarray_equiv<int64_t>(
-    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
-template void check_subarray_equiv<uint64_t>(
-    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
-template void check_subarray_equiv<float>(
-    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
-template void check_subarray_equiv<double>(
-    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
-
 template bool subarray_equiv<int8_t>(
     tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
 template bool subarray_equiv<uint8_t>(
