@@ -39,7 +39,6 @@
 #include <utility>
 #include <vector>
 
-#include "tiledb/common/logger_public.h"
 #include "tiledb/common/status.h"
 #include "tiledb/sm/array_schema/array_schema.h"
 #include "tiledb/sm/array_schema/dimension.h"
@@ -54,6 +53,7 @@
 #include "tiledb/sm/query/readers/aggregators/query_channel.h"
 #include "tiledb/sm/query/update_value.h"
 #include "tiledb/sm/query/validity_vector.h"
+#include "tiledb/sm/rest/rest_client.h"
 #include "tiledb/sm/storage_manager/storage_manager_declaration.h"
 #include "tiledb/sm/subarray/subarray.h"
 
@@ -62,6 +62,15 @@ using namespace tiledb::common;
 namespace tiledb {
 namespace sm {
 
+/** Class for query status exceptions. */
+class QueryException : public StatusException {
+ public:
+  explicit QueryException(const std::string& msg)
+      : StatusException("Query", msg) {
+  }
+};
+using QueryStatusException = QueryException;
+
 class Array;
 class ArrayDimensionLabelQueries;
 
@@ -162,36 +171,93 @@ class Query {
   /*                 API               */
   /* ********************************* */
 
+  /**
+   * Require that a name be that of a fixed-size field from the source array.
+   *
+   * Validate these conditions and throw if any are not met.
+   * - The name is that of a field in the array
+   * - The field is fixed-sized
+   *
+   * @param origin The name of the operation that this validation is a part of
+   * @param name The name of a field
+   */
+  void field_require_array_fixed(
+      const std::string_view origin, const char* name);
+
+  /**
+   * Require that a name be that of a variable-size field from the source array.
+   *
+   * Validate these conditions and throw if any are not met.
+   * - The name is that of a field in the array
+   * - The field is variable-sized
+   *
+   * @param origin The name of the operation that this validation is a part of
+   * @param name The name of a field
+   */
+  void field_require_array_variable(
+      const std::string_view origin, const char* name);
+
+  /**
+   * Require that a field be a nullable field from the source array.
+   *
+   * Validate these conditions and throw if any are not met.
+   * - The name is that of an attribute of the source array
+   * - The attribute is nullable
+   *
+   * @param origin The name of the operation that this validation is a part of
+   * @param name The name of a field
+   */
+  void field_require_array_nullable(
+      const std::string_view origin, const char* name);
+
+  /**
+   * Require that a field be a nonnull field from the source array.
+   *
+   * Validate these conditions and throw if any are not met.
+   * - The name is that of a field in the array
+   * - The field is not nullable
+   *
+   * @param origin The name of the operation that this validation is a part of.
+   * @param name The name of a field
+   */
+  void field_require_array_nonnull(
+      const std::string_view origin, const char* name);
+
   /**
    * Gets the estimated result size (in bytes) for the input fixed-sized
    * attribute/dimension.
    */
-  Status get_est_result_size(const char* name, uint64_t* size);
+  FieldDataSize get_est_result_size_fixed_nonnull(const char* name);
 
   /**
    * Gets the estimated result size (in bytes) for the input var-sized
    * attribute/dimension.
    */
-  Status get_est_result_size(
-      const char* name, uint64_t* size_off, uint64_t* size_val);
+  FieldDataSize get_est_result_size_variable_nonnull(const char* name);
 
   /**
    * Gets the estimated result size (in bytes) for the input fixed-sized,
    * nullable attribute.
    */
-  Status get_est_result_size_nullable(
-      const char* name, uint64_t* size_val, uint64_t* size_validity);
+  FieldDataSize get_est_result_size_fixed_nullable(const char* name);
 
   /**
    * Gets the estimated result size (in bytes) for the input var-sized,
    * nullable attribute.
    */
-  Status get_est_result_size_nullable(
-      const char* name,
-      uint64_t* size_off,
-      uint64_t* size_val,
-      uint64_t* size_validity);
+  FieldDataSize get_est_result_size_variable_nullable(const char* name);
 
+ private:
+  /**
+   * Common part of all `est_result_size_*` functions, called after argument
+   * validation.
+   *
+   * @param name The name of a field
+   * @return estimated result size
+   */
+  FieldDataSize internal_est_result_size(const char* name);
+
+ public:
   /** Retrieves the number of written fragments. */
   Status get_written_fragment_num(uint32_t* num) const;
 
@@ -655,6 +721,14 @@ class Query {
   /** Returns the internal stats object. */
   stats::Stats* stats() const;
 
+  /**
+   * Populate the owned stats instance with data.
+   * To be removed when the class will get a C41 constructor.
+   *
+   * @param data Data to populate the stats with.
+   */
+  void set_stats(const stats::StatsData& data);
+
   /** Returns the scratch space used for REST requests. */
   shared_ptr<Buffer> rest_scratch() const;
 
@@ -671,7 +745,7 @@ class Query {
       Layout layout, const ArraySchema& array_schema);
 
   /** Returns if all ranges for this query are non overlapping. */
-  tuple<Status, optional<bool>> non_overlapping_ranges();
+  bool non_overlapping_ranges();
 
   /** Returns true if this is a dense query */
   bool is_dense() const;
@@ -732,6 +806,21 @@ class Query {
   /** Returns true if the output field is an aggregate. */
   bool is_aggregate(std::string output_field_name) const;
 
+ private:
+  /**
+   * Create the aggregate channel object. This is split out because it's not
+   * at construction time, but on demand, and in two different situations.
+   */
+  void create_aggregate_channel() {
+    /*
+     * Because we have an extremely simple way of choosing channel identifiers,
+     * we can get away with hard-coding `1` here as the identifier for the
+     * aggregate channel.
+     */
+    aggregate_channel_ = make_shared<QueryChannel>(HERE(), *this, 1);
+  }
+
+ public:
   /**
    * Adds an aggregator to the default channel.
    *
@@ -740,6 +829,15 @@ class Query {
    */
   void add_aggregator_to_default_channel(
       std::string output_field_name, shared_ptr<IAggregator> aggregator) {
+    if (default_channel_aggregates_.empty()) {
+      /*
+       * Assert: this is the first aggregate added.
+       *
+       * We create the aggregate channel on demand, and this is when we need to
+       * do it.
+       */
+      create_aggregate_channel();
+    }
     default_channel_aggregates_.emplace(output_field_name, aggregator);
   }
 
@@ -750,20 +848,25 @@ class Query {
   /**
    * Get a list of all channels and their aggregates
    */
-  std::vector<QueryChannel> get_channels() {
+  std::vector<LegacyQueryAggregatesOverDefault> get_channels() {
     // Currently only the default channel is supported
-    return {QueryChannel(true, default_channel_aggregates_)};
+    return {
+        LegacyQueryAggregatesOverDefault(true, default_channel_aggregates_)};
   }
 
   /**
-   * Add a channel to the query
+   * Add a channel to the query. Used only by capnp serialization to initialize
+   * the aggregates list.
    */
-  void add_channel(const QueryChannel& channel) {
+  void add_channel(const LegacyQueryAggregatesOverDefault& channel) {
     if (channel.is_default()) {
       default_channel_aggregates_ = channel.aggregates();
+      if (!default_channel_aggregates_.empty()) {
+        create_aggregate_channel();
+      }
       return;
     }
-    throw std::logic_error(
+    throw QueryStatusException(
         "We currently only support a default channel for queries");
   }
 
@@ -772,14 +875,51 @@ class Query {
    */
   bool has_aggregates() {
     // We only need to check the default channel for now
-    return default_channel_aggregates_.empty();
+    return !default_channel_aggregates_.empty();
   }
 
+  /**
+   * Returns the number of channels.
+   *
+   * Responsibility for choosing channel identifiers is the responsibility of
+   * this class. At the present time the policy is very simple, since all
+   * queries only draw from a single array.
+   *   - Channel 0: All rows from the query. Always non-segmented, that is,
+   *       without any grouping.
+   *   - Channel 1: (optional) Simple aggregates, if any exist.
+   */
+  inline size_t number_of_channels() {
+    return has_aggregates() ? 1 : 0;
+  };
+
+  /**
+   * The default channel is initialized at construction and always exists.
+   */
+  inline std::shared_ptr<QueryChannel> default_channel() {
+    return default_channel_;
+  }
+
+  inline std::shared_ptr<QueryChannel> aggegate_channel() {
+    if (!has_aggregates()) {
+      throw QueryStatusException("Aggregate channel does not exist");
+    }
+    return aggregate_channel_;
+  }
+
+  /**
+   * Returns the REST client configured in the storage manager associated to
+   * this query
+   */
+  RestClient* rest_client() const;
+
  private:
   /* ********************************* */
   /*         PRIVATE ATTRIBUTES        */
   /* ********************************* */
 
+  /** The query memory tracker. */
+  shared_ptr<MemoryTracker> query_memory_tracker_;
+
   /** A smart pointer to the array the query is associated with.
    * Ensures that the Array object exists as long as the Query object exists. */
   shared_ptr<Array> array_shared_;
@@ -957,6 +1097,25 @@ class Query {
   std::unordered_map<std::string, shared_ptr<IAggregator>>
       default_channel_aggregates_;
 
+  /*
+   * Handles to channels use shared pointers, so the channels are allocated here
+   * for ease of implementation.
+   *
+   * At present there's only one possible non-default channel, so we keep track
+   * of it in its own variable. A fully C.41 class might simply store these in
+   * a constant vector.
+   */
+  /**
+   * The default channel is allocated in the constructor for simplicity.
+   */
+  std::shared_ptr<QueryChannel> default_channel_;
+
+  /**
+   * The aggegregate channel is optional, so we initialize it as empty with it
+   * default constructor.
+   */
+  std::shared_ptr<QueryChannel> aggregate_channel_{};
+
   /* ********************************* */
   /*           PRIVATE METHODS         */
   /* ********************************* */
