@@ -89,6 +89,7 @@ UnorderedWriter::UnorderedWriter(
           remote_query,
           fragment_name)
     , frag_uri_(std::nullopt)
+    , cell_pos_(query_memory_tracker_->get_resource(MemoryType::WRITER_DATA))
     , written_buffers_(written_buffers)
     , is_coords_pass_(true) {
   // Check the layout is unordered.
@@ -167,7 +168,7 @@ std::string UnorderedWriter::name() {
 
 Status UnorderedWriter::alloc_frag_meta() {
   // Alloc FragmentMetadata object.
-  frag_meta_ = make_shared<FragmentMetadata>(HERE());
+  frag_meta_ = this->create_fragment_metadata();
   // Used in serialization when FragmentMetadata is built from ground up.
   frag_meta_->set_context_resources(&storage_manager_->resources());
 
@@ -372,7 +373,7 @@ Status UnorderedWriter::compute_coord_dups() {
 }
 
 Status UnorderedWriter::prepare_tiles(
-    std::unordered_map<std::string, WriterTileTupleVector>* tiles) const {
+    tdb::pmr::unordered_map<std::string, WriterTileTupleVector>* tiles) const {
   auto timer_se = stats_->start_timer("prepare_tiles");
 
   // Initialize attribute tiles
@@ -380,7 +381,10 @@ Status UnorderedWriter::prepare_tiles(
   for (const auto& it : buffers_) {
     const auto& name = it.first;
     if (written_buffers_.count(name) == 0) {
-      (*tiles).emplace(name, WriterTileTupleVector());
+      tiles->emplace(
+          std::piecewise_construct,
+          std::forward_as_tuple(name),
+          std::forward_as_tuple(query_memory_tracker_));
     }
   }
 
@@ -389,8 +393,8 @@ Status UnorderedWriter::prepare_tiles(
       storage_manager_->compute_tp(), 0, tiles->size(), [&](uint64_t i) {
         auto tiles_it = tiles->begin();
         std::advance(tiles_it, i);
-        const auto& name = tiles_it->first;
-        RETURN_CANCEL_OR_ERROR(prepare_tiles(name, &((*tiles)[name])));
+        RETURN_CANCEL_OR_ERROR(
+            prepare_tiles(tiles_it->first, &(tiles_it->second)));
         return Status::Ok();
       });
 
@@ -427,8 +431,14 @@ Status UnorderedWriter::prepare_tiles_fixed(
   // Initialize tiles
   tiles->reserve(tile_num);
   for (uint64_t i = 0; i < tile_num; i++) {
-    tiles->emplace_back(WriterTileTuple(
-        array_schema_, cell_num_per_tile, false, nullable, cell_size, type));
+    tiles->emplace_back(
+        array_schema_,
+        cell_num_per_tile,
+        false,
+        nullable,
+        cell_size,
+        type,
+        query_memory_tracker_);
   }
 
   // Write all cells one by one
@@ -498,8 +508,14 @@ Status UnorderedWriter::prepare_tiles_var(
   // Initialize tiles
   tiles->reserve(tile_num);
   for (uint64_t i = 0; i < tile_num; i++) {
-    tiles->emplace_back(WriterTileTuple(
-        array_schema_, cell_num_per_tile, true, nullable, cell_size, type));
+    tiles->emplace_back(
+        array_schema_,
+        cell_num_per_tile,
+        true,
+        nullable,
+        cell_size,
+        type,
+        query_memory_tracker_);
   }
 
   // Write all cells one by one
@@ -653,14 +669,15 @@ Status UnorderedWriter::unordered_write() {
     }
 
     // Create new fragment
-    frag_meta_ = make_shared<FragmentMetadata>(HERE());
+    frag_meta_ = this->create_fragment_metadata();
     RETURN_CANCEL_OR_ERROR(create_fragment(false, frag_meta_));
   }
 
   frag_uri_ = frag_meta_->fragment_uri();
 
   // Prepare tiles
-  std::unordered_map<std::string, WriterTileTupleVector> tiles;
+  tdb::pmr::unordered_map<std::string, WriterTileTupleVector> tiles(
+      query_memory_tracker_->get_resource(MemoryType::WRITER_TILE_DATA));
   RETURN_CANCEL_OR_ERROR(prepare_tiles(&tiles));
 
   // No tiles
